<?php

namespace Drupal\Tests\snowball_stemmer\Kernel;

use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\KernelTests\KernelTestBase;
use Drupal\snowball_stemmer\Stemmer;

/**
 * Tests the search_excerpt() function.
 *
 * @group snowball_stemmer
 */
class CoreSearchTest extends KernelTestBase {

  /**
   * Stemmer service mock.
   *
   * @var \Drupal\snowball_stemmer\Stemmer
   */
  protected Stemmer $stemmerService;

  /**
   * Language manager mock.
   *
   * @var \Drupal\Core\LanguageManagerInterface
   */
  protected LanguageManagerInterface $languageManager;

  /**
   * Modules to enable.
   *
   * @var array
   */
  protected static $modules = [
    'search',
    'snowball_stemmer',
  ];

  /**
   * Creates a new processor object for use in the tests.
   */
  protected function setUp(): void {
    parent::setUp();

    $this->stemmerService = $this->createMock(Stemmer::class);
    $this->container->set('snowball_stemmer.stemmer', $this->stemmerService);
    $this->languageManager = $this->createMock(LanguageManagerInterface::class);
    $this->container->set('language_manager', $this->languageManager);
    \Drupal::setContainer($this->container);
  }

  /**
   * Tests search_simplify() and the stemmer hook_search_preprocess integration.
   */
  public function testSearchSimplify() {
    $this->stemmerService
      ->expects($this->once())
      ->method('setLanguage')
      ->with('en')
      ->willReturn(TRUE);
    $matcher = $this->exactly(4);

    $this->stemmerService
      ->expects($matcher)
      ->method('stem')->willReturnCallback(function (...$parameters) use ($matcher) {
        if ($matcher->numberOfInvocations() === 1) {
            $this->assertSame('the', $parameters[0]);
        }
        if ($matcher->numberOfInvocations() === 2) {
            $this->assertSame('quick', $parameters[0]);
        }
        if ($matcher->numberOfInvocations() === 3) {
            $this->assertSame('brown', $parameters[0]);
        }
        if ($matcher->numberOfInvocations() === 4) {
            $this->assertSame('fox', $parameters[0]);
        }
    });

    $text = 'The quick brown fox';
    $language = 'en';
    \Drupal::service('search.text_processor')->analyze($text, $language);
  }

  /**
   * Tests the hook stemming alone.
   */
  public function testSearchStemming() {
    // HTML is stripped by Search module, but non-alpha-numeric characters are
    // maintained for later tokenizing. Search simplify has lowercased the
    // string.
    $text = 'van de groep 65-plussers is dat 14%.';
    $language = 'nl';
    $expected = 'van de groep 65-plusser is dat 14%.';

    $this->stemmerService
      ->expects($this->once())
      ->method('setLanguage')
      ->with('nl')
      ->willReturn(TRUE);

    $matcher = $this->exactly(8);

    $this->stemmerService
      ->expects($matcher)
      ->method('stem')->willReturnCallback(function (...$parameters) use ($matcher) {
        if ($matcher->numberOfInvocations() === 1) {
          $this->assertSame('van', $parameters[0]);
          return 'van';
        }
        if ($matcher->numberOfInvocations() === 2) {
          $this->assertSame('de', $parameters[0]);
          return 'de';
        }
        if ($matcher->numberOfInvocations() === 3) {
          $this->assertSame('groep', $parameters[0]);
          return 'groep';
        }
        if ($matcher->numberOfInvocations() === 4) {
          $this->assertSame('65', $parameters[0]);
          return '65';
        }
        if ($matcher->numberOfInvocations() === 5) {
          $this->assertSame('plussers', $parameters[0]);
          return 'plusser';
        }
        if ($matcher->numberOfInvocations() === 6) {
          $this->assertSame('is', $parameters[0]);
          return 'is';
        }
        if ($matcher->numberOfInvocations() === 7) {
          $this->assertSame('dat', $parameters[0]);
          return 'dat';
        }
        if ($matcher->numberOfInvocations() === 8) {
          $this->assertSame('14', $parameters[0]);
          return '14';
        }
      });

    $out = snowball_stemmer_search_preprocess($text, $language);
    $this->assertEquals($out, $expected);
  }

}
